-- thanks to sweetman for contributing parts of the race store code
races = {}
diff = {}
maxLevel = {}
racescount = 0
penalties = 0
level = 1
currentrace = ""
trackpenalty = false
cache_file = getSetting("Cache Path") .. "race_scores_ladoga2.dat"
mapName = "Ladoga"


function writeScore()
    -- save all content deleting the first one
    local f = io.open(cache_file, "w")
    if f == nil then
        log('race score file could not be opened to write scores')
    else
        f:write(level)
        for i,r in ipairs(iraces) do
            f:write("\n"..r[1].."|"..races[r[1]]["besttime"])
        end
        f:close()
        log('race score file written')
    end
end

function readlevel()
    local f = io.open(cache_file, "r")
    if f == nil then
        log("could not read cache_file")
    else
        level = f:read("*n")
        if level == nil then level = 1 end
        f:close()
    end
end


-- return a string:
-- (+3 min 2 sec)
-- (-34 sec)
-- <empty string>
function formatPartialTime(sec)
    local asec = math.abs(sec)
    local m = math.modf (asec / 60)
    local s = math.modf (asec % 60)
    st = string.format("%02.0f",s).." sec"
    if m > 0 then
        st = string.format("%2.0f", m)..":"..st
    end
    if sec > 0.0 then st = " (+"..st..")"
    elseif sec < 0.0 then st = " (-"..st..")"
    else st = ""
    end
    return st
end

function checkPenalty(racename, atchk)
    if not races[racename]['usepenalization'] then return end
    if table.getn(races[racename]['penalties']) >= atchk then
        if races[racename]['penalties'][atchk] then return    end
    end
    races[racename]['penalties'][atchk] = true
    penalties = races[racename]['penaltytime'] + penalties
    trackpenalty = false
end

function formatSeconds(sec)
    local m = math.modf (sec / 60)
    local s = math.modf (sec % 60)
    st = string.format("%02.0f",s).." sec"
    if m > 0 then
        st = string.format("%2.0f", m)..":"..st
    end
    return st
end

function findScore(racename)
    minval = 9999
    local f = io.open(cache_file, "r")
    if f == nil then return minval end
    for line in f:lines() do
        line_ar = explode('|', line)
        if table.getn(line_ar) > 1 then
            if line_ar[1] == racename and tonumber(line_ar[2]) < minval then
                minval = tonumber(line_ar[2])
            end
        end
    end
    f:close()
    log('race score file read')
    return minval
end

function resetRace()
    setupArrow(-1)
    -- reset some values
    if currentrace ~= "" then
        races[currentrace]['checkpointspassed'] = 0
        races[currentrace]['penalties'] = {}
		stopTimer()
    end
    currentrace = ""
    penalties = 0
end

function showInfo(hardness)
	-- This function indicates the user which race to run when crossing a large gate (green/orange/red)
	diff[0] = "easy"
	diff[1] = "medium"
	diff[2] = "hard"
	maxLevel[0] = 3
	maxLevel[1] = 6
	maxLevel[2] = 9
	racesCompleted = level - 1 - maxLevel[hardness] + 3
	if racesCompleted < 0 then racesCompleted = 0 end
	-- if all races in the level have been completed tell the user to try next level
	if level > maxLevel[hardness] then
		if hardness == 3 then
			msg = diff[hardness].." level\nAll races completed, congratulations! Post your times on Rigs on Rods forum!"
		else
			msg = diff[hardness].." level\nAll races completed, congratulations! Try "..diff[hardness + 1].." level."
		end
	-- if no races of this level have been unlocked tell the user which race to complete
	elseif level < maxLevel[hardness] - 2 then
		msg = diff[hardness].." level - locked.\nWin golden medals for all races in "..diff[hardness - 1].." level to unlock."
	-- tell the user how many races he completed  in the present difficulty level
	else
		msg = diff[hardness].." level\nRaces completed: "..racesCompleted.."/3"
	end
    flashMessage(msg, 4)
end

function cheat()
	flashMessage("aw", 4)
end

function loadCheckpoints(race)
    for a,v in ipairs(races[race]['positions']) do
        if a > 1 then
        -- first checkpoint is already spawn
            iname = 'race_'..race..'_'..a
            log("spawn chk number "..a.." of race "..race.." odef file "..v[7].." luaName "..iname)
            spawnObject(v[7], iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
        end
    end
end


    --SetupRace has several parameters:
    --  racename to show on screen (no spaces nor underscore)
    --  array of checkpoints positions
    -- is loop ?
    --  Print Altitude of the next checpoint on screen ?
function setupRace(racename, positions, loop, showaltitude)
    log("setup race "..racename)
    racescount = racescount + 1
    if table.getn(positions) > 1 and racescount <= level then
		spawnObject(positions[1][7], 'race_'..racename..'_1', positions[1][1],positions[1][2],positions[1][3], positions[1][4],positions[1][5],positions[1][6], raceEventHandler)
		log("spawned checkpoint number 1 of race "..racename.." odef file "..positions[1][7].." luaName "..'race_'..racename..'_1' )
    end
    -- add the race finally
    races[racename] = {}
    races[racename]['racename'] = racename
    races[racename]['positions'] = positions
    races[racename]['isloop'] = loop
    races[racename]['startTime'] = 0
    races[racename]['lastcheckpoint'] = 1
    races[racename]['checkpointspassed'] = 1
    races[racename]['numcheckpoints'] = table.getn(positions)
    races[racename]['checkpointsloaded'] = false
    races[racename]['showaltitude'] = showaltitude
    races[racename]['usepenalization'] = false
    races[racename]['penaltytime'] = 0.0
    races[racename]['penalties'] = {}
    races[racename]["goldentime"] = 0.0
    races[racename]["platetime"] = 0.0
    races[racename]["bronzetime"] = 0.0
    races[racename]["idxrace"] = racescount
    races[racename]['besttime'] = findScore(racename)
    if loop then
        races[racename]['goalnum'] = 1
    else
        races[racename]['goalnum'] = table.getn(positions)
    end

end

function nextRace(something)
    for a,v in ipairs(iraces) do
        if races[v[1]]["idxrace"] == something then
            local iname = 'race_'..v[1]..'_1'
            local position = races[v[1]]["positions"][1]
            spawnObject(position[7], iname, position[1],position[2],position[3], position[4],position[5],position[6], raceEventHandler)
            log("spawn chk number 1 of race "..v[1].." odef file "..position[7].." luaName "..iname)
			return
        end
    end
end


function getMedal(racename, spent)
    if spent <= races[racename]["goldentime"] then
        return "Golden"
    elseif spent <= races[racename]["platetime"] then
        return "Plate"
    elseif spent <= races[racename]["bronzetime"] then
        return "Bronze"
    else
        return "\t"
    end
end

function getCheckPointFromName(instance)
    ar = explode('_', instance)
    if table.getn(ar) < 3 then return -1
    else
        return tonumber(ar[3])
    end
end

function getRacenameFromName(instance)
    ar = explode('_', instance)
    if table.getn(ar) < 2 then return ""
    else
        return tostring(ar[2])
    end
end

function setupArrow(position)
    if position == -1 or position == nil then
        -- hide it
        pointArrow(0, 0, 0, "")
    else
        txt = currentrace.."\ncheckpoint "..tostring(position).." / "..races[currentrace]['numcheckpoints']
        if position == 1 and races[currentrace]['isloop'] then
            txt = "Finish"
        end
        data = races[currentrace]['positions'][position]
        if data then
            if races[currentrace]["showaltitude"] then
                txt = txt .. "\n\nAlt "..string.format("%.0f", races[currentrace]['positions'][position][2] * 3,2808399 ).." feets"
            end
            pointArrow(data[1], data[2], data[3], txt)
        else
            pointArrow(0, 0, 0, "")
        end
    end
end

function raceEventHandler(instance, box)
    checknum = getCheckPointFromName(instance)
    racename = getRacenameFromName(instance)

    if box == "resetcurrentrace"  then
        resetRace()
    elseif box == "penalty" then
        trackpenalty = true
		flashMessage("PENALTY!",2)
	elseif box == "showinfogreen" then
		showInfo(0)
	elseif box == "showinfoorange" then
		showInfo(1)
	elseif box == "showinfored" then
		showInfo(2)
	elseif box == "cheat" then
		showInfo(0)
    else
        -- create an array with all required names in it
        if currentrace == "" then
            -- not racing currently
            if checknum == 1 then
                -- start of the race
                currentrace = racename
                races[currentrace]['startTime'] = getTime()
                races[currentrace]['lastcheckpoint'] = 1
                races[currentrace]['checkpointspassed'] = 1
                if not races[racename]['checkpointsloaded'] then
                    loadCheckpoints(racename)
                    races[racename]['checkpointsloaded'] = true
                end
                if trackpenalty then checkPenalty(racename, checknum) end
                setupArrow(2)
                startTimer()
                msg = "Welcome to "..mapName.." Championship\nRace "..races[currentrace]["idxrace"].." of ".. racescount.. " started!"
                flashMessage(msg, 4)
            end
        elseif currentrace == racename then
            -- we hit a checkpoint from the same race!
            if checknum == races[currentrace]['lastcheckpoint'] + 1 then
                -- racing, correct checkpoint!
                if checknum == races[currentrace]['numcheckpoints'] and races[currentrace]['isloop'] then
                    setupArrow(1)
                else
                    setupArrow(checknum+1)
                end
                races[currentrace]['lastcheckpoint'] = checknum
                races[currentrace]['checkpointspassed'] = races[currentrace]['checkpointspassed'] + 1
                if trackpenalty then checkPenalty(racename, checknum) end
                time = getTime() - races[currentrace]['startTime']
                msg = "Completed: "..checknum.."/"..(races[currentrace]['numcheckpoints']).." in "..formatSeconds(time)
                flashMessage(msg, 4)
            else
                if not checknum == races[currentrace]['lastcheckpoint'] then
                    -- racing, wrong checkpoint, prevent trigger of the same checkpoint twice
                    flashMessage("Wrong gate! You must find and pass Checkpoint "..races[currentrace]['lastcheckpoint'] + 1, 4)
                end
            end
            if checknum == races[currentrace]['goalnum'] and races[currentrace]['checkpointspassed'] == races[racename]['numcheckpoints'] then
                -- racing, reached finish
                --time = getTime() - starttime
                line1 = ""
                line2 = ""
                time = stopTimer()
                if races[racename]['usepenalization']  then
                    line1 = "penalty \t\t\t "..line1 --one more tab than line2
                    line2 = formatSeconds(penalties).."\t\t "
                end
                line1 = "Medal \t\ttime\t\t".. line1.."final time"
                line2 = getMedal(racename, time).. "\t\t"..formatSeconds(time).."\t\t"..line2..formatSeconds(time + penalties)
                time = time + penalties
                local savescores = false
                if time < races[currentrace]["goldentime"] and level < 9 then
                    level = level + 1
                    nextRace(level)
                    savescores = true
                end
                if time < tonumber(races[currentrace]['besttime']) then
                    line2 = line2 .. "\nNew Best Lap Time!"
                    if tonumber(races[currentrace]['besttime']) ~= 9999 then
                        line2 = line2.. formatPartialTime(time - races[currentrace]['besttime'])
                    end
                    races[currentrace]['besttime'] = time
                    savescores = true
                    if time < races[currentrace]["goldentime"] and level < 9 then
                        line2 = line2.."\n Unlocked next race, good work!"
                    end
                else
                    line2 = line2 .. "\n (Best Lap: "..formatSeconds(races[currentrace]['besttime'])..")"
                end
                if savescores then writeScore() end
                log(currentrace.." finished. Score:\n"..line1.."\n"..line2)
                flashMessage(line1.."\n"..line2, 15)
                resetRace()
            end
        else
            -- flashMessage("This Checkpoint belongs to race"..racename.."!", 4)
        end
    end
end

------------------------------------------------------------------------------------------------------------
--array of checkpoints  to use: x, y, z, rotX, rotY, rotZ
race_test_1 = {
{53.608154,  3.129692, 977.335205,  0.000000, -7.000000,  0.000000, "dgchk"},
{103.673317,  2.484437, 778.462158,  0.000000, -53.500000,  0.000000, "dgchk"},
{197.883423,  2.891431, 781.104370,  0.000000, 41.500000,  0.000000, "dgchk"},
{248.932312,  3.194018, 877.070190,  0.000000, 43.000000,  0.000000, "dgchk"},
{336.029236,  5.318645, 858.200317,  0.400000, 32.000000,  0.300000, "dgchk"},
{380.023254,  3.536777, 890.712158,  0.000000, 53.000000,  0.000000, "dgchk"},
{409.925720,  4.157577, 913.465210, -0.500000, 40.500000,  0.000000, "dgchk"},
{553.593506,  4.180667, 976.690613,  0.000000, 64.500000,  0.000000, "dgchk"},
{727.360779,  4.026832, 920.630066,  0.000000, -43.500000,  0.000000, "dgchk"},
{897.756531,  3.892306, 674.279724,  0.000000, -29.000000,  0.000000, "dgchk"},
{937.637878,  3.701584, 549.273682,  0.000000,  5.000000,  0.000000, "dgchk"},
{907.230408,  4.297204, 511.807434,  0.000000, 59.500000,  0.000000, "dgchk"},
{837.498169,  4.966671, 552.517761,  0.000000, -21.500000,  0.000000, "dgchk"},
{821.275330,  4.739823, 585.384338,  0.000000, -19.000000,  0.000000, "dgchk"},
{762.690125,  3.741343, 589.012756,  0.000000, -74.500000,  0.000000, "dgchk"},
{697.116089,  3.981638, 613.819336,  0.000000, -63.500000,  0.000000, "dgchk"},
{641.695618,  3.779227, 676.957275,  0.000000, -30.000000,  0.000000, "dgchk"},
{564.708191,  4.450894, 837.003357,  0.000000, -51.500000,  0.000000, "dgchk"},
{492.973206,  3.824628, 766.489929,  0.000000, 87.500000,  0.000000, "dgchk"},
{373.656006,  4.048292, 697.207031,  0.000000, 41.000000,  0.000000, "dgchk"},
{289.994507,  5.636880, 649.202454,  0.000000, -53.500000,  0.000000, "dgchk"},
{260.952209,  6.243996, 653.611206,  0.000000,  -128.500000,  0.000000, "dgchk"},
{217.114914, 10.565752, 653.987976,  0.000000, -10.000000,  0.000000, "dgchk"},
{175.685211,  9.880802, 694.290894,  0.000000,  -110.500000,  0.000000, "dgchk"},
{155.300522, 10.095018, 678.907410,  0.000000,  0.000000,  0.000000, "dgchk"},
{176.173111, 14.933770, 648.919739,  0.000000, -31.500000,  0.000000, "dgchk"},
{147.654770, 23.808573, 598.391724,  0.000000, 84.500000,  0.000000, "dgchk"},
{118.234871, 28.882782, 646.567261,  0.000000,  0.000000,  0.000000, "dgchk"},
{93.998337, 18.361364, 706.278076,  0.000000, -50.500000,  0.000000, "dgchk"},
{56.376308, 16.267023, 692.942322,  0.000000, 17.000000,  0.000000, "dgchk"},
{61.971405, 29.438530, 613.492310,  0.000000, 37.500000,  0.000000, "dgchk"},
{143.507156,  8.344702, 511.134888,  0.000000, -54.500000,  0.000000, "dgchk"},
{183.388580,  9.479033, 523.579102,  0.000000, 48.500000,  0.000000, "dgchk"},
{218.082275, 10.688625, 494.322144,  0.000000, -14.000000,  0.000000, "dgchk"},
{265.531952,  8.684541, 445.744232,  0.000000, 73.000000,  0.000000, "dgchk"},
{361.448059,  4.271160, 335.455048,  0.000000, -27.000000,  0.000000, "dgchk"},
{381.072540,  4.587872, 297.541809,  0.000000, -22.500000,  0.000000, "dgchk"},
{459.192078,  3.772445, 228.207611,  0.000000, -29.000000,  0.000000, "dgchk"},
{498.979156,  3.811145, 186.987869,  0.000000, -30.500000,  0.000000, "dgchk"},
{568.943298,  5.077972, 100.188110,  0.000000, -84.000000,  0.000000, "dgchk"},
{667.768433,  3.625344, 83.044067,  0.000000, 93.500000,  0.500000, "dgchk"},
{766.985107,  4.464178, 36.293976,  0.000000, 106.500000,  0.000000, "dgchk"},
{845.648682,  5.039272, 42.760925,  0.000000, 141.500000,  0.000000, "dgchk"},
{846.136536,  5.398427, 132.639420,  0.000000, 38.000000,  0.000000, "dgchk"},
{892.270081,  4.134654, 170.353394, -0.200000, 64.000000, -0.400000, "dgchk"},
{882.306030,  4.219862, 232.948700,  0.000000, -44.500000,  0.000000, "dgchk"},
{680.772888,  4.600705, 311.294983,  0.000000, -50.000000,  0.000000, "dgchk"},
{556.865967,  4.202209, 367.726318,  0.000000, -89.500000,  0.000000, "dgchk"},
{367.517242,  5.040980, 420.135773,  0.000000, -41.000000,  0.000000, "dgchk"},
{344.350616,  4.440451, 434.463593,  0.000000, -51.000000,  0.000000, "dgchk"},
{281.785034,  3.877213, 366.124603,  0.000000,  0.000000,  0.000000, "dgchk"},
{263.210815,  3.278340, 317.100464,  0.000000, 97.000000,  0.000000, "dgchk"},
{241.584320,  5.830740, 296.564240,  0.000000, 88.500000,  0.000000, "dgchk"},
{218.920975,  4.806921, 276.412109,  0.000000,  -133.500000,  0.000000, "dgchk"},
{215.120377,  5.835194, 234.779663,  0.000000, -59.000000,  0.000000, "dgchk"},
{262.975067,  5.304360, 151.529892,  0.000000, 24.000000,  0.000000, "dgchk"},
{209.115112,  6.274483, 46.012997,  0.000000, -86.000000,  0.000000, "dgchk"},
{153.299866, 23.237276, 82.109634,  0.000000, -36.500000,  0.000000, "dgchk"},
{115.345230, 29.468973, 110.372002,  0.000000, -24.000000,  0.000000, "dgchk"},
{106.557487, 28.975544, 143.866531,  0.000000,  0.000000,  0.000000, "dgchk"},
{134.546616, 26.637278, 168.371735,  0.000000, 60.500000,  0.000000, "dgchk"},
{184.249054,  7.618290, 208.957672,  0.000000, -26.000000,  0.000000, "dgchk"},
{108.697861, 14.977919, 237.359940,  0.000000,  0.000000,  0.000000, "dgchk"}
}

race_test_2={
{257.241699, 3.747055,   329.251221, 0.000000, 0.000000, 0.000000, "dgchk"},
{255.215179, 4.723322,   341.306915, 0.000000, 0.000000, 0.000000, "dgchk"}
}


function init()
    readlevel()
    -- spawn Stage pole that inform user what race is the next one.
    -- spawnObject("dgstartgreen", "dgstartgreen", 728.514771, 5.741645, 610.253540, 0.000000, 122.000000, -1.000000, raceEventHandler)
    -- spawnObject("dgstartorange", "dgstartorange", 749.136047, 6.025445, 626.739990, 0.000000, 167.000000, -1.000000, raceEventHandler)
    -- spawnObject("dgstartred", "dgstartred", 777.517944, 6.287702, 614.447998, 0.000000, 215.000000, -1.000000, raceEventHandler)
    --spawnObject(oname, iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
    -- 1
    local race = "Test1"
    setupRace(race, race_test_1, false, false)
	races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 20.0
    races[race]["goldentime"] = 286.07
    races[race]["platetime"]  = races[race]["goldentime"] + 60
    races[race]["bronzetime"] = races[race]["platetime"] + 60
    log("LUA Test 1 loaded")
    -- 2
    race = "Test2"
    setupRace(race, race_test_2, false, false)
    races[race]["usepenalization"] = true
    races[race]["penaltytime"] = 60.0
    races[race]["goldentime"] = 240.01
    races[race]["platetime"]  = races[race]["goldentime"] + 60
    races[race]["bronzetime"] = races[race]["platetime"] + 60

    log("LUA Test 2 loaded")
    iraces = {
			{"Test1"},
            {"Test2"}
    }

end

init()
log("Ladoga lua file loaded")
